# coding:utf-8
version = "0.0.6"
import logging
import json
import consul


class Base(object):
    __handers__ = {}
    _type_dict_ = {}

    def __init__(self, cfg, consul_host="127.0.0.1", consul_port=8500):
        self.cfg = cfg
        self.consul_host = consul_host
        self.consul_port = consul_port
        self.init()

    def add_change_callback(self, *keys, callback_handler=None):
        if not keys or not callback_handler:
            return
        for key in keys:
            if key and key not in self.__handers__:
                self.__handers__[key] = callback_handler
        self.watching()

    def watching(self):
        pass

    def init(self):
        c = consul.Consul(self.consul_host, self.consul_port)
        cfg_items = None
        if not isinstance(self.cfg, dict):
            cfg_items = self.cfg.__class__.__dict__.items()
        else:
            cfg_items = self.cfg.items()
        self._type_dict_ = {k: type(v) for k, v in cfg_items if k[:1] != "_"}
        _, data = c.kv.get(self.cfg.__base_key__, recurse=True)
        if isinstance(data, list):
            self.fill_config(data)

    def fill_config(self, data):
        for item in data:
            r, key = item.get("Key", "").split("/", 1)
            if r != self.cfg.__base_key__:
                continue
            if key not in self._type_dict_:
                logging.debug("key:%s not define at options. passed!", key)
                continue
            if self._type_dict_[key] is None:
                continue
            keyType = self._type_dict_[key]
            if (
                keyType is list
                or keyType is tuple
                or keyType is dict
                or keyType is bool
            ):
                v = json.loads(item["Value"])
            elif keyType is str:
                v = item["Value"].decode()
            else:
                v = keyType(item["Value"])
            if isinstance(self.cfg, dict):
                self.cfg[key] = v
            else:
                setattr(self.cfg, key, v)
            logging.debug("CFG:key:%s,value:%s", key, v)




# coding:utf-8

import asyncio
import os
import logging
import json

import consul.aio
from consul.base import Timeout

from .. import Base

class DynamicPatch(Base):
    _conn_ = None
    _loop_ = None

    def __init__(self, cfg, consul_host="127.0.0.1", consul_port=8500):
        super().__init__(cfg,consul_host=consul_host,consul_port=consul_port)
        self._loop_ = asyncio.get_event_loop()
        self._conn_ = consul.aio.Consul(port=consul_port, loop=self._loop_)

    def start(self):
        self._loop_.run_forever()
        self._loop_.close()

    def watching(self):
        for key, callback_handler in self.__handers__.items():
            self._loop_.create_task(self.watch(key, callback_handler))

    @asyncio.coroutine
    def watch(self, key, callback_handler):
        index = None
        while True:
            try:
                index, data = yield from self._conn_.kv.get(
                    os.path.join(self.cfg.__base_key__, key), index=index
                )
                if data is not None:
                    if hasattr(self.cfg, key):
                        keyType = self._type_dict_[key]
                        if keyType is list or keyType is dict or keyType is bool:
                            v= json.loads(data["Value"])
                        elif keyType is str:
                            v = data["Value"].decode()
                        else:
                            v = keyType(data["Value"])
                        if getattr(self.cfg, key) != v:
                            setattr(self.cfg, key, v)
                            callback_handler(key,v)
            except asyncio.TimeoutError:
                pass




#!/usr/bin/env python
# coding:utf-8
#
# Copyright (c) 2019 xtqxk All rights reserved.
#
# @author: XTQXK <xtqxk.kilroy@gmail.com>
# Created on SEP 2, 2019
#

try:
    # Use setuptools if available, for install_requires (among other things).
    import setuptools
    from setuptools import setup
except ImportError:
    setuptools = None
    from distutils.core import setup

version = "0.0.6"

setup(
    name="consul-owl",
    version=version,
    packages=["owl", "owl.aio", "owl.gevent", "owl.tornado"],
    author="XTQXK",
    author_email="xtqxk.kilroy@gmail.com",
    url="https://github.com/xtqxk/owl",
    license="http://www.apache.org/licenses/LICENSE-2.0",
    description="Owl is a python Consul config center for Tornado, Gevent, Flask and AioHTTP",
    install_requires=["python-consul"],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
)
